#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Windows.Forms;

namespace HeuristicLab.Optimization.Networks.Views {
  [View("InputOutputPort View")]
  [Content(typeof(InputOutputPort<,>), true)]
  [Content(typeof(IInputOutputPort<,>), false)]
  [Content(typeof(IInputOutputPort), false)]
  public partial class InputOutputPortView : ValuePortView {
    protected EntitySelectorDialog entitySelectorDialog;

    public new IInputOutputPort Content {
      get { return (IInputOutputPort)base.Content; }
      set { base.Content = value; }
    }

    public InputOutputPortView() {
      InitializeComponent();
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (entitySelectorDialog != null) entitySelectorDialog.Dispose();
        if (components != null) components.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void DeregisterContentEvents() {
      Content.OutputInputPortChanged -= Content_OutputInputPortChanged;
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.OutputInputPortChanged += Content_OutputInputPortChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      outputInputPortView.Content = Content == null ? null : Content.OutputInputPort;
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      outputInputPortGroupBox.Enabled = Content != null && !ReadOnly;
      setOutputInputPortButton.Enabled = Content != null && !ReadOnly;
      clearOutputInputPortButton.Enabled = Content != null && Content.OutputInputPort != null && !ReadOnly;
    }

    protected virtual void Content_OutputInputPortChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_OutputInputPortChanged), sender, e);
      else {
        clearOutputInputPortButton.Enabled = Content.OutputInputPort != null && !ReadOnly;
        outputInputPortView.Content = Content.OutputInputPort;
      }
    }

    protected virtual void setOutputInputPortButton_Click(object sender, EventArgs e) {
      if (entitySelectorDialog == null) {
        entitySelectorDialog = new EntitySelectorDialog();
        entitySelectorDialog.Caption = "Select Output/Input Port";
        entitySelectorDialog.EntitySelector.Caption = "Available Output/Input Ports";
      }

      try {
        IEntity root = Content;
        while (root.Parent != null)
          root = root.Parent;
        entitySelectorDialog.EntitySelector.Configure(
          root,
          Content.OutputInputPort,
          typeof(IOutputInputPort<,>).MakeGenericType(Content.GetType().GetGenericArguments()[0], Content.GetType().GetGenericArguments()[1])
        );

        if (entitySelectorDialog.ShowDialog(this) == DialogResult.OK) {
          Content.OutputInputPort = entitySelectorDialog.EntitySelector.SelectedEntity as IOutputInputPort;
        }
      }
      catch (Exception ex) {
        ErrorHandling.ShowErrorDialog(this, ex);
      }
    }
    protected virtual void clearOutputInputPortButton_Click(object sender, EventArgs e) {
      Content.OutputInputPort = null;
    }

    protected virtual void outputInputPortView_DragEnterOver(object sender, DragEventArgs e) {
      e.Effect = DragDropEffects.None;
      var data = e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat);
      var type = typeof(IOutputInputPort<,>).MakeGenericType(Content.GetType().GetGenericArguments()[0], Content.GetType().GetGenericArguments()[1]);

      if (!ReadOnly && (type.IsAssignableFrom(data.GetType()))) {
        e.Effect = DragDropEffects.Link;
      }
    }
    protected virtual void outputInputPortView_DragDrop(object sender, DragEventArgs e) {
      if (e.Effect != DragDropEffects.None) {
        IOutputInputPort port = e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) as IOutputInputPort;
        Content.OutputInputPort = port;
      }
    }
  }
}
